//--------------------------- BASIC PROPERTIES ------------------------------

#include "include\\Warnings.fxh"

float4x4 World;
float4x4 View;
float4x4 Projection;
float4x4 WorldInverseTranspose;

static const int MAX_MATRICES = 35;
float4x4 WorldMatrixArray[MAX_MATRICES] : WORLDMATRIXARRAY;
int numBoneInfluences = 2;

bool Skinning;
bool SelfShadows;

float3 DiffuseLightDirection;
float4 DiffuseColor = float4(1, 1, 1, 1);
float DiffuseIntensity = 1.0;
float4 LineColor = float4(0, 0, 0, 1);
float LineThickness;

texture Texture;
 
// The texture sampler, which will get the texture color
sampler2D textureSampler = sampler_state
{
    Texture = (Texture);
    Filter = COMPARISON_MIN_MAG_LINEAR_MIP_POINT;
    AddressU = Clamp;
    AddressV = Clamp;
};
 
//--------------------------- DATA STRUCTURES ------------------------------
struct VS_INPUT
{
    float4 Position : POSITION0;            // The position of the vertex
    float3 Normal : NORMAL0;                // The vertex's normal
    float2 TextureCoordinate : TEXCOORD0;    // The texture coordinate of the vertex
	float4 BlendWeights : BLENDWEIGHT0;		//
    float4 BlendIndices : BLENDINDICES0;	//
};
struct VS_OUTPUT
{
    float4 Position : POSITION0;
    float2 TextureCoordinate : TEXCOORD0;
    float3 Normal : TEXCOORD1;
};
 
//--------------------------- SHADERS ------------------------------

// The vertex shader that does the outlines
VS_OUTPUT VShader(VS_INPUT input)
{
    VS_OUTPUT output = (VS_OUTPUT) 0;

    float4 p = float4(0.0f, 0.0f, 0.0f, 1.0f);
    float3 norm = float3(0.0f, 0.0f, 0.0f);
    int n = numBoneInfluences - 1;
    input.Normal = normalize(input.Normal);

    if (Skinning)
    {
		// cast the vectors to arrays for use in the for loop below
        float BlendWeightsArray[4] = (float[4]) input.BlendWeights;

        if (1)
        {
            float lastWeight = 0.0f;
    
			//Blend vertex position & normal
            for (int i = 0; i < n; ++i)
            {
                lastWeight += BlendWeightsArray[i];
                p += BlendWeightsArray[i] * mul(input.Position, WorldMatrixArray[input.BlendIndices[i]]);
                norm += BlendWeightsArray[i] * mul(input.Normal, WorldMatrixArray[input.BlendIndices[i]]);
            }
            lastWeight = 1.0f - lastWeight;
    
            p += lastWeight * mul(input.Position, WorldMatrixArray[input.BlendIndices[n]]);
            norm += lastWeight * mul(input.Normal, WorldMatrixArray[input.BlendIndices[n]]);
            p.w = 1.0f;
        }
        else
        {
			// calculate the pos/normal
            for (int i = 0; i < n; ++i)
            {
                p += BlendWeightsArray[i] * mul(input.Position, WorldMatrixArray[input.BlendIndices[i]]);
                norm += BlendWeightsArray[i] * mul(input.Normal, WorldMatrixArray[input.BlendIndices[i]]);
            }
        }
    }
    else
    {
        p = mul(input.Position, World);
        norm = mul(input.Normal, World);
    }
 
    // Calculate where the vertex ought to be.  This line is equivalent
    // to the transformations in the CelVertexShader.
    float4  v_original = mul(mul(p, View), Projection);
 
    // Calculates the normal of the vertex like it ought to be.
    float4 normal = mul(mul(norm, View), Projection);
    normal = normalize(normal);
 
    // Take the correct "original" location and translate the vertex a little
    // bit in the direction of the normal to draw a slightly expanded object.
    // Later, we will draw over most of this with the right color, except the expanded
    // part, which will leave the outline that we want.
    output.Position =  v_original + (mul(LineThickness, normal));
 
    return output;
}

// The pixel shader for the outline.  It is pretty simple:  draw everything with the
// correct line color.
float4 PShader(VS_OUTPUT input) : SV_Target0
{
    return LineColor;
}